"use strict";

var path = require('path');

var marker = /^ *@deferred *$/;
var importSpecifiers = ['ImportDefaultSpecifier', 'ImportSpecifier', 'ImportNamespaceSpecifier'];

module.exports = function (babel) {
  var template = babel.template,
      t = babel.types;
  var buildImport = template('DECLARATION; require([MODULE], function(ARG) { ASSIGNMENTS })');
  return {
    visitor: {
      ImportDeclaration: function ImportDeclaration(nodepath) {
        if (!(nodepath.node.trailingComments || []).some(function (c) {
          return marker.test(c.value);
        })) {
          return;
        }

        var specs = nodepath.node.specifiers.map(function (spec) {
          if (!importSpecifiers.includes(spec.type) || spec.local.type !== 'Identifier') {
            throw Error('Unexpected format for import specifier');
          }

          return spec;
        });
        var ARG = '_';

        while (specs.some(function (spec) {
          return spec.local.name === ARG;
        })) {
          ARG += '_';
        }

        var MODULE = t.StringLiteral(path.normalize(path.join(path.dirname(this.getModuleName()), nodepath.node.source.value)));
        var DECLARATION = t.VariableDeclaration('let', specs.map(function (spec) {
          return t.VariableDeclarator(t.Identifier(spec.local.name));
        }));
        var ASSIGNMENTS = specs.map(function (spec) {
          return t.ExpressionStatement(t.AssignmentExpression('=', t.Identifier(spec.local.name), spec.type === 'ImportNamespaceSpecifier' ? t.Identifier(ARG) : t.MemberExpression(t.Identifier(ARG), t.Identifier(spec.type === 'ImportDefaultSpecifier' ? 'default' : spec.local.name))));
        });
        var replace = buildImport({
          DECLARATION: DECLARATION,
          ARG: ARG,
          ASSIGNMENTS: ASSIGNMENTS,
          MODULE: MODULE
        });
        nodepath.replaceWithMultiple(replace);
      }
    }
  };
};